@@ -22,21 +22,26 @@ from label_studio_tools.core.utils.params import get_env
 logger = logging.getLogger(__name__)
 
 _LABEL_TAGS = {'Label', 'Choice'}
-_NOT_CONTROL_TAGS = {'Filter',}
-LOCAL_FILES_DOCUMENT_ROOT = get_env('LOCAL_FILES_DOCUMENT_ROOT', default=os.path.abspath(os.sep))
+_NOT_CONTROL_TAGS = {
+    'Filter',
+}
+LOCAL_FILES_DOCUMENT_ROOT = get_env(
+    'LOCAL_FILES_DOCUMENT_ROOT', default=os.path.abspath(os.sep)
+)
 
 TreebankWordTokenizer.PUNCTUATION = [
-        (re.compile(r"([:,])([^\d])"), r" \1 \2"),
-        (re.compile(r"([:,])$"), r" \1 "),
-        (re.compile(r"\.\.\."), r" ... "),
-        (re.compile(r"[;@#$/%&]"), r" \g<0> "),
-        (
-            re.compile(r'([^\.])(\.)([\]\)}>"\']*)\s*$'),
-            r"\1 \2\3 ",
-        ),  # Handles the final period.
-        (re.compile(r"[?!]"), r" \g<0> "),
-        (re.compile(r"([^'])' "), r"\1 ' "),
-    ]
+    (re.compile(r"([:,])([^\d])"), r" \1 \2"),
+    (re.compile(r"([:,])$"), r" \1 "),
+    (re.compile(r"\.\.\."), r" ... "),
+    (re.compile(r"[;@#$/%&]"), r" \g<0> "),
+    (
+        re.compile(r'([^\.])(\.)([\]\)}>"\']*)\s*$'),
+        r"\1 \2\3 ",
+    ),  # Handles the final period.
+    (re.compile(r"[?!]"), r" \g<0> "),
+    (re.compile(r"([^'])' "), r"\1 ' "),
+]
+
 
 class ExpandFullPath(argparse.Action):
     def __call__(self, parser, namespace, values, option_string=None):
@@ -56,24 +61,33 @@ def tokenize(text):
 
 
 def create_tokens_and_tags(text, spans):
-    #tokens_and_idx = tokenize(text) # This function doesn't work properly if text contains multiple whitespaces...
-    token_index_tuples = [token for token in TreebankWordTokenizer().span_tokenize(text)]
+    # tokens_and_idx = tokenize(text) # This function doesn't work properly if text contains multiple whitespaces...
+    token_index_tuples = [
+        token for token in TreebankWordTokenizer().span_tokenize(text)
+    ]
     tokens_and_idx = [(text[start:end], start) for start, end in token_index_tuples]
-    if spans and all([span.get('start') is not None and span.get('end') is not None for span in spans]):
+    if spans and all(
+        [
+            span.get('start') is not None and span.get('end') is not None
+            for span in spans
+        ]
+    ):
         spans = list(sorted(spans, key=itemgetter('start')))
         span = spans.pop(0)
         span_start = span['start']
-        span_end = span['end']-1
+        span_end = span['end'] - 1
         prefix = 'B-'
         tokens, tags = [], []
         for token, token_start in tokens_and_idx:
             tokens.append(token)
-            token_end = token_start + len(token) - 1#"- 1" - This substraction is wrong. token already uses the index E.g. "Hello" is 0-4
-            token_start_ind = token_start  #It seems like the token start is too early.. for whichever reason
+            token_end = (
+                token_start + len(token) - 1
+            )  # "- 1" - This substraction is wrong. token already uses the index E.g. "Hello" is 0-4
+            token_start_ind = token_start  # It seems like the token start is too early.. for whichever reason
 
-            #if for some reason end of span is missed.. pop the new span (Which is quite probable due to this method)
-            #Attention it seems like span['end'] is the index of first char afterwards. In case the whitespace is part of the
-            #labell we need to subtract one. Otherwise next token won't trigger the span update.. only the token after next..
+            # if for some reason end of span is missed.. pop the new span (Which is quite probable due to this method)
+            # Attention it seems like span['end'] is the index of first char afterwards. In case the whitespace is part of the
+            # labell we need to subtract one. Otherwise next token won't trigger the span update.. only the token after next..
             if token_start_ind > span_end:
                 while spans:
                     span = spans.pop(0)
@@ -110,12 +124,21 @@ def _get_upload_dir(project_dir=None, upload_dir=None):
         if not os.path.exists(upload_dir):
             upload_dir = None
     if not upload_dir:
-        raise FileNotFoundError("Can't find upload dir: either LS_UPLOAD_DIR or project should be passed to converter")
+        raise FileNotFoundError(
+            "Can't find upload dir: either LS_UPLOAD_DIR or project should be passed to converter"
+        )
     return upload_dir
 
 
-def download(url, output_dir, filename=None, project_dir=None, return_relative_path=False, upload_dir=None,
-             download_resources=True):
+def download(
+    url,
+    output_dir,
+    filename=None,
+    project_dir=None,
+    return_relative_path=False,
+    upload_dir=None,
+    download_resources=True,
+):
     is_local_file = url.startswith('/data/') and '?d=' in url
     is_uploaded_file = url.startswith('/data/upload')
 
@@ -123,11 +146,17 @@ def download(url, output_dir, filename=None, project_dir=None, return_relative_p
         upload_dir = _get_upload_dir(project_dir, upload_dir)
         filename = urllib.parse.unquote(url.replace('/data/upload/', ''))
         filepath = os.path.join(upload_dir, filename)
-        logger.debug(f'Copy {filepath} to {output_dir}'.format(filepath=filepath, output_dir=output_dir))
+        logger.debug(
+            f'Copy {filepath} to {output_dir}'.format(
+                filepath=filepath, output_dir=output_dir
+            )
+        )
         if download_resources:
             shutil.copy(filepath, output_dir)
         if return_relative_path:
-            return os.path.join(os.path.basename(output_dir), os.path.basename(filename))
+            return os.path.join(
+                os.path.basename(output_dir), os.path.basename(filename)
+            )
         return filepath
 
     if is_local_file:
@@ -145,7 +174,14 @@ def download(url, output_dir, filename=None, project_dir=None, return_relative_p
         filename = f'{basename}{ext}'
         filepath = os.path.join(output_dir, filename)
         if os.path.exists(filepath):
-            filename = basename + '_' + hashlib.md5(url.encode() + str(datetime.datetime.now().timestamp()).encode()).hexdigest()[:4] + ext
+            filename = (
+                basename
+                + '_'
+                + hashlib.md5(
+                    url.encode() + str(datetime.datetime.now().timestamp()).encode()
+                ).hexdigest()[:4]
+                + ext
+            )
 
     filepath = os.path.join(output_dir, filename)
     if not os.path.exists(filepath):
@@ -183,19 +219,19 @@ def ensure_dir(dir_path):
 
 def parse_config(config_string):
     """
-        :param config_string: Label config string
-        :return: structured config of the form:
-        {
-            "<ControlTag>.name": {
-                "type": "ControlTag",
-                "to_name": ["<ObjectTag1>.name", "<ObjectTag2>.name"],
-                "inputs: [
-                    {"type": "ObjectTag1", "value": "<ObjectTag1>.value"},
-                    {"type": "ObjectTag2", "value": "<ObjectTag2>.value"}
-                ],
-                "labels": ["Label1", "Label2", "Label3"] // taken from "alias" if exists or "value"
-        }
-        """
+    :param config_string: Label config string
+    :return: structured config of the form:
+    {
+        "<ControlTag>.name": {
+            "type": "ControlTag",
+            "to_name": ["<ObjectTag1>.name", "<ObjectTag2>.name"],
+            "inputs: [
+                {"type": "ObjectTag1", "value": "<ObjectTag1>.value"},
+                {"type": "ObjectTag2", "value": "<ObjectTag2>.value"}
+            ],
+            "labels": ["Label1", "Label2", "Label3"] // taken from "alias" if exists or "value"
+    }
+    """
     if not config_string:
         return {}
 
@@ -203,7 +239,11 @@ def parse_config(config_string):
         return tag.attrib.get('name') and tag.attrib.get('value')
 
     def _is_output_tag(tag):
-        return tag.attrib.get('name') and tag.attrib.get('toName') and tag.tag not in _NOT_CONTROL_TAGS
+        return (
+            tag.attrib.get('name')
+            and tag.attrib.get('toName')
+            and tag.tag not in _NOT_CONTROL_TAGS
+        )
 
     def _get_parent_output_tag_name(tag, outputs):
         # Find parental <Choices> tag for nested tags like <Choices><View><View><Choice>...
@@ -231,14 +271,23 @@ def parse_config(config_string):
                 if tag.attrib.get('whenTagName'):
                     conditionals = {'type': 'tag', 'name': tag.attrib['whenTagName']}
                 elif tag.attrib.get('whenLabelValue'):
-                    conditionals = {'type': 'label', 'name': tag.attrib['whenLabelValue']}
+                    conditionals = {
+                        'type': 'label',
+                        'name': tag.attrib['whenLabelValue'],
+                    }
                 elif tag.attrib.get('whenChoiceValue'):
-                    conditionals = {'type': 'choice', 'name': tag.attrib['whenChoiceValue']}
+                    conditionals = {
+                        'type': 'choice',
+                        'name': tag.attrib['whenChoiceValue'],
+                    }
             if conditionals:
                 tag_info['conditionals'] = conditionals
             outputs[tag.attrib['name']] = tag_info
         elif _is_input_tag(tag):
-            inputs[tag.attrib['name']] = {'type': tag.tag, 'value': tag.attrib['value'].lstrip('$')}
+            inputs[tag.attrib['name']] = {
+                'type': tag.tag,
+                'value': tag.attrib['value'].lstrip('$'),
+            }
         if tag.tag not in _LABEL_TAGS:
             continue
         parent_name = _get_parent_output_tag_name(tag, outputs)
@@ -247,7 +296,9 @@ def parse_config(config_string):
             if not actual_value:
                 logger.debug(
                     'Inspecting tag {tag_name}... found no "value" or "alias" attributes.'.format(
-                        tag_name=etree.tostring(tag, encoding='unicode').strip()[:50]))
+                        tag_name=etree.tostring(tag, encoding='unicode').strip()[:50]
+                    )
+                )
             else:
                 labels[parent_name][actual_value] = dict(tag.attrib)
     for output_tag, tag_info in outputs.items():
@@ -256,7 +307,8 @@ def parse_config(config_string):
             if input_tag_name not in inputs:
                 logger.debug(
                     f'to_name={input_tag_name} is specified for output tag name={output_tag}, '
-                    'but we can\'t find it among input tags')
+                    'but we can\'t find it among input tags'
+                )
                 continue
             tag_info['inputs'].append(inputs[input_tag_name])
         tag_info['labels'] = list(labels[output_tag])
@@ -280,8 +332,7 @@ def get_polygon_bounding_box(x, y):
 
 
 def _get_annotator(item, default=None, int_id=False):
-    """ Get annotator id or email from annotation
-    """
+    """Get annotator id or email from annotation"""
     annotator = item['completed_by']
     if isinstance(annotator, dict):
         annotator = annotator.get('email', default)
